/********************************************************\
*	ErrorLib - Error dtection library for fx-9860G	*
*	ErrorLib is free software					*
**********************************************************
*	Author: Hugo "LIMachi" Martzolf			*
*	Contact: lee.mach.hm@gmail.com			*
**********************************************************
*	file ErrorLib.c						*
*	Include header for ErrorLib				*
**********************************************************
*	date 21-10-2012						*
\********************************************************/

#include "ErrorLib.h"
#include <stdlib.h>
#include <stdio.h>

/*************************************\
*	Dependencies management	*
\*************************************/

#ifdef ERROR
#define REBOOT
#endif

/******************\
*	Functions	*
\******************/

#ifdef REBOOT

//Thanks to Simon Lothar, Andreas B and CasioCalc.org for this syscall code
//Use this function to reboot the calculator

char* ReBoot(void)
{
	int SysCallCode[] = {0xD201422B,0x60F20000,0x80010070};
	int (*SysCall)( int R4, int R5, int R6, int R7, int FNo ) = (void*)&SysCallCode;

	return (char*)((*SysCall)(0, 0, 0, 0, 0x236));
}
#endif

#ifdef ERROR

//this function pop-up an error window if the code it's lower than 0
// the utilisator can chose what he do when the error ocure

int Error(int code)
{
	unsigned char *textError, codeError[10];
	unsigned int key=0;

	if(code<0)
	{
		sprintf(codeError,"ERROR: %d",code);

		if(code>-44)
		{
			switch(code)
			{
				case -1:
					textError=malloc(16);
					memcpy(textError,"Entry Not Found",16);
				break;
				case -2:
					textError=malloc(17);
					memcpy(textError,"Illegal Parameter",17);
				break;
				case -3:
					textError=malloc(12);
					memcpy(textError,"Illegal Path",12);
				break;
				case -4:
					textError=malloc(11);
					memcpy(textError,"Device Full",11);
				break;
				case -5:
					textError=malloc(14);
					memcpy(textError,"Illegal Device",14);
				break;
				case -6:
					textError=malloc(18);
					memcpy(textError,"Illegal FileSystem",18);
				break;
				case -7:
					textError=malloc(14);
					memcpy(textError,"Illegal System",14);
				break;
				case -8:
					textError=malloc(12);
					memcpy(textError,"Acces Denyed",12);
				break;
				case -9:
					textError=malloc(14);
					memcpy(textError,"Already Locked",14);
				break;
				case -10:
					textError=malloc(15);
					memcpy(textError,"Illegal Task ID",15);
				break;
				case -11:
					textError=malloc(15);
					memcpy(textError,"Permision Error",15);
				break;
				case -12:
					textError=malloc(10);
					memcpy(textError,"Entry Full",10);
				break;
				case -13:
					textError=malloc(19);
					memcpy(textError,"Already Exist Entry",19);
				break;
				case -14:
					textError=malloc(14);
					memcpy(textError,"Read Only File",14);
				break;
				case -15:
					textError=malloc(14);
					memcpy(textError,"Illegal Filter",14);
				break;
				case -16:
					textError=malloc(13);
					memcpy(textError,"Enum Rate End",13);
				break;
				case -17:
					textError=malloc(14);
					memcpy(textError,"Device Changed",14);
				break;
				case -18:
					textError=malloc(15);
					memcpy(textError,"Not Record File",15);
				break;
				case -19:
					textError=malloc(16);
					memcpy(textError,"Illegal Seek Pos",16);
				break;
				case -20:
					textError=malloc(18);
					memcpy(textError,"Illegal Block File",18);
				break;
				case -21:
					textError=malloc(16);
					memcpy(textError,"Device Not Exist",16);
				break;
				case -22:
					textError=malloc(11);
					memcpy(textError,"End Of File",11);
				break;
				case -23:
					textError=malloc(16);
					memcpy(textError,"Not Mount Device",16);
				break;
				case -24:
					textError=malloc(18);
					memcpy(textError,"Not Unmount Device",18);
				break;
				case -25:
					textError=malloc(18);
					memcpy(textError,"Cannot Lock System",18);
				break;
				case -26:
					textError=malloc(16);
					memcpy(textError,"Record Not Found",16);
				break;
				case -27:
					textError=malloc(20);
					memcpy(textError,"Not Dual Record File",20);
				break;
				case -28:
					textError=malloc(17);
					memcpy(textError,"Not Alarm Support",17);
				break;
				case -29:
					textError=malloc(16);
					memcpy(textError,"Cannot Add Alarm",16);
				break;
				case -30:
					textError=malloc(14);
					memcpy(textError,"File Find Used",14);
				break;
				case -31:
					textError=malloc(12);
					memcpy(textError,"Device Error",12);
				break;
				case -32:
					textError=malloc(17);
					memcpy(textError,"System Not Locked",17);
				break;
				case -33:
					textError=malloc(16);
					memcpy(textError,"Device Not Found",16);
				break;
				case -34:
					textError=malloc(18);
					memcpy(textError,"File Type Mismatch",18);
				break;
				case -35:
					textError=malloc(9);
					memcpy(textError,"Not Empty",9);
				break;
				case -36:
					textError=malloc(19);
					memcpy(textError,"Brocken System Data",19);
				break;
				case -37:
					textError=malloc(15);
					memcpy(textError,"Media Not Ready",15);
				break;
				case -38:
					textError=malloc(19);
					memcpy(textError,"Too Many Alarm Item",19);
				break;
				case -39:
					textError=malloc(16);
					memcpy(textError,"Same Alarm Exist",16);
				break;
				case -40:
					textError=malloc(17);
					memcpy(textError,"Acces S Swap Area",17);
				break;
				case -41:
					textError=malloc(16);
					memcpy(textError,"Multi Media Card",16);
				break;
				case -42:
					textError=malloc(15);
					memcpy(textError,"Copy Protection",15);
				break;
				case -43:
					textError=malloc(17);
					memcpy(textError,"Illegal File Data",17);
				break;
			}
		}
		else
		{
			textError=malloc(13);
			memcpy(textError,"Unknown Error",13);
		}

		PopUpWin(5);
		locate(3,2);
		Print(codeError);
		locate(3,3);
		Print(textError);
		locate(3,4);
		Print("EXE: continue");
		locate(3,5);
		Print("MENU: exit");
		locate(3,6);
		Print("EXIT: reboot");

		while(key!=30004)
		{
			GetKey(&key);
			if(key==30002) ReBoot();
		}
	}
}
#endif